@@ -1,5 +1,8 @@
+import typing
+
 from construct import Construct, Container
 
+from mercury_engine_data_structures import crc
 from mercury_engine_data_structures.game_check import Game
 
 
@@ -30,3 +33,10 @@ class BaseResource:
 
 AssetType = str
 AssetId = int
+NameOrAssetId = typing.Union[str, AssetId]
+
+
+def resolve_asset_id(value: NameOrAssetId) -> AssetId:
+    if isinstance(value, str):
+        return crc.crc64(value)
+    return value
