@@ -3,6 +3,7 @@ from pathlib import Path
 import pytest
 from construct.lib.containers import Container
 
+from mercury_engine_data_structures.file_tree_editor import FileTreeEditor
 from mercury_engine_data_structures.game_check import Game
 
 
@@ -41,3 +42,14 @@ def parse_and_build_compare_parsed(module, game: Game, file_path: Path, print_da
 def purge_hidden(data: Container) -> Container:
     data = {k: v for k, v in data.items() if not k.startswith("_")}
     return {k: purge_hidden(v) if isinstance(v, Container) else v for k, v in data.items()}
+
+
+def parse_build_compare_editor(module, editor: FileTreeEditor, file_name: str, print_data=False):
+    raw = editor.get_raw_asset(file_name)
+
+    data = module.parse(raw, target_game=editor.target_game)
+    if print_data:
+        print(data)
+    encoded = module.build(data, target_game=editor.target_game)
+
+    assert encoded == raw
